import tamber

def basic_test():
    print("event/track\n")

    try:
        e = tamber.Event.track(
            user='user_jctzgisbru', 
            item='item_i5gq90scc1', 
            behavior='mention',
            hit=True,
            context={
                "page": "detail-view",
                "section": "recommended"
            }
        )
        print(e)
    except tamber.TamberError as e:
        print(e)


    print("\ndiscover/recommended\n")

    try:
        d = tamber.Discover.recommended(user='user_jctzgisbru',number=10)
        print(d)
        for rec in d:
            print("item:%s  score:%s\n" % (rec['item'],rec['score']))
    except tamber.TamberError as e:
        print(e)

    print("\ndiscover/next\n")

    try:
        d = tamber.Discover.next(user='user_jctzgisbru',item='item_i5gq90scc1',number=20)
        print(d)
        for rec in d:
            print("item:%s  score:%s\n" % (rec['item'],rec['score']))
    except tamber.TamberError as e:
        print(e)

def partial_test():
    try:
        print(tamber.Item.create(
            id='item_faa666arma',
            properties={
                'clothing_type': 'pants',
                'stock':         90,
            },
            tags=['casual', 'feminine']
        ))
    except tamber.TamberError as e:
        print(e)

    try:
        print(tamber.Item.retrieve(
            id='item_faa666arma',
        ))
    except tamber.TamberError as e:
        print(e)

    try:
        print(tamber.Item.update(
            id='item_faa666arma',
            updates={
                'add': {
                    'properties': {'stock': 89}
                }
            }
        ))
    except tamber.TamberError as e:
        print(e)

    # try:
    #     print(tamber.User.search(filter={'age': 30}))
    # except tamber.TamberError as e:
    #     print(e)


print("Running Tests...")

tamber.project_key = 'Mu6DUPXdDYe98cv5JIfX'
tamber.engine_key = 'SbWYPBNdARfIDa0IIO9L'

print("Basic Test:\n")

basic_test()

print("Partial Test:\n")

partial_test()

print("\n\n=================\nTesting Complete\n=================\n\n")



from tamber import api_url, api
import json
import warnings

def deprecated(func):
    """This is a decorator which can be used to mark functions
    as deprecated. It will result in a warning being emmitted
    when the function is used."""
    def newFunc(*args, **kwargs):
        warnings.warn("Call to deprecated function %s." % func.__name__,
                      category=DeprecationWarning)
        return func(*args, **kwargs)
    newFunc.__name__ = func.__name__
    newFunc.__doc__ = func.__doc__
    newFunc.__dict__.update(func.__dict__)
    return newFunc

class TamberObject():
    def __init__(self, *args, **kwargs):
        super(TamberObject, self).__init__(*args, **kwargs)

class TamberJSONEncoder(json.JSONEncoder):
    def default(self, obj):
        if not isinstance(obj, TamberObject):
            return super(TamberJSONEncoder, self).default(obj)
        return obj.__dict__

class APIResource():
    @staticmethod
    def _get_url_path(resource, command):
        return "/%s/%s" % (resource, command)

    @classmethod
    def _url_path(cls, method):
        return cls._get_url_path(cls.name, method)

    @classmethod
    def _flatten_args(cls, keys=None, **params):
        for k in params:
            if isinstance(params[k], dict) or isinstance(params[k], list) or isinstance(params[k], TamberObject):
                params[k] = json.dumps(params[k], cls=TamberJSONEncoder)
        if keys:
            reserved = {'api_url', 'project_key', 'engine_key', 'timeout'}
            return dict((k, params[k]) for k in params if k in keys or k in reserved)
        return params

    @classmethod
    def _call_api(cls, method, url, keys=None, **params):
        args = cls._flatten_args(keys, **params)
        return api.call_api(method, url, **args)

# item, user, behavior
class CreateableAPIResource(APIResource):
    @classmethod
    def create(cls, **params):
        url = cls._url_path('create')
        return cls._call_api('POST', url, **params)

    @classmethod
    def retrieve(cls, **params):
        url = cls._url_path('retrieve')
        return cls._call_api('GET', url, **params)

# item, user
class UpdatableAPIResource(APIResource):
    @classmethod
    def update(cls, **params):
        url = cls._url_path('update')
        return cls._call_api('POST', url, **params)

class ListableAPIResource(APIResource):
    @classmethod
    def list(cls, **params):
        url = cls._url_path('list')
        return cls._call_api('POST', url, **params)

# event/track, user/create, user/update, user/retrieve
class GetRecs(TamberObject):
    def __init__(self, number=None, page=None, exclude_items=None, variability=None, filter=None, get_properties=None):
        self.number = number
        self.page = page
        self.exclude_items = exclude_items
        self.variability = variability
        self.filter = filter
        self.get_properties = get_properties

class Event(APIResource, TamberObject):
    name = 'event'
    def __init__(self, user=None, item=None, behavior=None, amount=None, hit=False, context=None, created=None):
        self.user = user
        self.item = item
        self.behavior = behavior
        self.amount = amount
        self.hit = hit
        self.context = context
        self.created = created

    @classmethod
    def track(cls, **params):
        keys = {'user', 'item', 'behavior', 'amount', 'hit', 'context', 'created'}
        return cls._call_api('POST', cls._url_path('track'), keys, **params)

    @classmethod
    def retrieve(cls, **params):
        keys = {'user', 'item', 'behavior', 'created_since', 'created_before', 'number'}
        if 'before' in params:
            params['created_before'] = params['before']
        if 'since' in params:
            params['created_since'] = params['since']
        return cls._call_api('POST', cls._url_path('retrieve'), keys, **params)
    
    @classmethod
    def batch(cls, **params):
        keys = {'events'}
        return cls._call_api('POST', cls._url_path('batch'), keys, **params)

    # Meta methods
    @classmethod
    def metaLike(cls, **params):
        keys = {'user', 'property', 'value', 'amount'}
        return cls._call_api('POST', cls._url_path('meta/like'), keys, **params)

    @classmethod
    def metaUnlike(cls, **params):
        keys = {'user', 'property', 'value'}
        return cls._call_api('POST', cls._url_path('meta/unlike'), keys, **params)


class Discover(APIResource):
    name = 'discover'

    @classmethod
    def recommended(cls, **params):
        keys = {'user', 'number', 'exclude_items', 'variability', 'filter', 'get_properties', 'continuation', 'no_create'}
        return cls._call_api('GET', cls._url_path('recommended'), keys, **params)

    @classmethod
    def next(cls, **params):
        keys = {'user', 'item', 'number', 'exclude_items', 'variability', 'filter', 'get_properties', 'continuation', 'no_create'}
        return cls._call_api('GET', cls._url_path('next'), keys, **params)

    @classmethod
    def weekly(cls, **params):
        keys = {'user', 'number', 'exclude_items', 'filter', 'get_properties', 'no_create'}
        return cls._call_api('GET', cls._url_path('popular'), keys, **params)

    @classmethod
    def daily(cls, **params):
        keys = {'user', 'number', 'exclude_items', 'filter', 'get_properties', 'no_create'}
        return cls._call_api('GET', cls._url_path('popular'), keys, **params)

    @classmethod
    def meta(cls, **params):
        keys = {'property', 'user', 'item', 'number', 'variability', 'no_create'}
        return cls._call_api('GET', cls._url_path('popular'), keys, **params)

    @classmethod
    def popular(cls, **params):
        keys = {'number', 'page', 'filter', 'get_properties'}
        return cls._call_api('GET', cls._url_path('popular'), keys, **params)

    @classmethod
    def hot(cls, **params):
        keys = {'number', 'page', 'filter', 'get_properties'}
        return cls._call_api('GET', cls._url_path('hot'), keys, **params)

    @classmethod
    def uac(cls, **params):
        keys = {'number', 'page', 'filter', 'get_properties'}
        return cls._call_api('GET', cls._url_path('uac'), keys, **params)

    @classmethod
    def userPopular(cls, **params):
        keys = {'number', 'page', 'filter', 'get_properties', 'user', 'no_create'}
        return cls._call_api('GET', cls._url_path('user_trend/popular'), keys, **params)

    @classmethod
    def userHot(cls, **params):
        keys = {'number', 'page', 'filter', 'get_properties', 'user', 'no_create'}
        return cls._call_api('GET', cls._url_path('user_trend/hot'), keys, **params)

    @classmethod
    def userUac(cls, **params):
        keys = {'number', 'page', 'filter', 'get_properties', 'user', 'no_create'}
        return cls._call_api('GET', cls._url_path('user_trend/uac'), keys, **params)

    @classmethod
    def basicRecommended(cls, **params):
        keys = {'user', 'number', 'page', 'filter', 'test_events', 'get_properties'}
        return cls._call_api('GET', cls._url_path('basic/recommended'), keys, **params)
        
    @classmethod
    def basicSimilar(cls, **params):
        keys = {'item', 'number', 'page', 'filter', 'test_events', 'get_properties'}
        return cls._call_api('GET', cls._url_path('basic/similar'), keys, **params)

    @classmethod
    def basicRecommendedSimilar(cls, **params):
        keys = {'user', 'item', 'number', 'page', 'filter', 'test_events', 'get_properties'}
        return cls._call_api('GET', cls._url_path('basic/recommended_similar'), keys, **params)


class User(CreateableAPIResource, UpdatableAPIResource, ListableAPIResource):
    name = 'user'

    @classmethod
    @deprecated
    def search(cls, **params):
        keys = {'filter'}
        return cls._call_api('GET', cls._url_path('list'), keys, **params)

    @classmethod
    def merge(cls, **params):
        keys = {'from', 'to', 'no_create'}
        return cls._call_api('POST', cls._url_path('merge'), keys, **params)

class Item(CreateableAPIResource, UpdatableAPIResource, ListableAPIResource):
    name = 'item'
    
    @classmethod
    def hide(cls, **params):
        url = cls._url_path('hide')
        keys = {'id'}
        return cls._call_api('POST', url, **params)

    @classmethod
    def unhide(cls, **params):
        url = cls._url_path('unhide')
        keys = {'id'}
        return cls._call_api('POST', url, **params)

    @classmethod
    def delete(cls, **params):
        url = cls._url_path('delete')
        keys = {'id'}
        return cls._call_api('POST', url, **params)

class Behavior(CreateableAPIResource):
    name = 'behavior'



import sys

class TamberError(Exception):
    def __init__(self, message=None, is_timed_out_error=False):
        super(TamberError, self).__init__(message)
        self._message = message
        self.is_timed_out_error = is_timed_out_error



import json

class Filter(dict):
    def __init__(self, function=None, *args):
        if function == 'property':
            self[function] = args[0]
        else:
            self[function] = list(args)

    def __str__(self):
        return json.dumps(self)

And = lambda *args: Filter('and', *args)
Or = lambda *args: Filter('or', *args)
Gt = lambda *args: Filter('gt', *args)
Gte = lambda *args: Filter('gte', *args)
Lt = lambda *args: Filter('lt', *args)
Lte = lambda *args: Filter('lte', *args)
Eq = lambda *args: Filter('eq', *args)
Neq = lambda *args: Filter('neq', *args)
Not = lambda arg: Filter('not', arg)
Contains = lambda *args: Filter('contains', *args)
Overlaps = lambda *args: Filter('overlaps', *args)
Property = lambda property: Filter('property', property)
Tags = lambda tags: Filter('tags', *[])


import sys
import json
import base64
import tamber

PYTHON_VERSION = sys.version_info[0]

if PYTHON_VERSION == 2:
    from urllib2 import urlopen, Request, HTTPError, URLError
    from urllib import urlencode
    b64encode = lambda s: base64.b64encode(s.encode('ascii'))
elif PYTHON_VERSION == 3:
    from urllib.request import urlopen, Request
    from urllib.error import HTTPError, URLError
    from urllib.parse import urlencode
    b64encode = lambda s: base64.b64encode(s.encode('ascii')).decode('ascii')

def call_api(method, url, api_url=None, project_key=None, engine_key=None, timeout=None, **params):
    if api_url is None:
        api_url = tamber.get_api_url()
    if project_key is None:
        project_key = tamber.get_project_key()
    if engine_key is None:
        engine_key = tamber.get_engine_key()
    if timeout is None:
        timeout = params.get('timeout') or tamber.get_timeout()

    url = api_url + url
    encoded_params = urlencode(params)
    data = None
    headers = dict()

    if method == 'POST':
        headers['Content-Type'] = 'application/x-www-form-urlencoded'
        data = encoded_params
        if PYTHON_VERSION == 3:
            data = data.encode("utf-8")
    elif method == 'GET':
        url += '?' + encoded_params

    api_version = tamber.get_api_version()
    if api_version != None:
        headers['Tamber-Version'] = api_version
    headers['User-Agent'] = 'Tamber/v1 PythonBindings/%s' % tamber.version_str()
    headers['Authorization'] = 'Basic %s' % b64encode(project_key+':'+engine_key)

    req = Request(url, data=data, headers=headers)

    try:
        resp = urlopen(req, timeout=timeout)
    except HTTPError as e:
        resp = e
    except URLError as e:
        raise tamber.error.TamberError(
            message="Unexpected timeout error. If this problem persists, let us know at support@tamber.com.", 
            is_timed_out_error=True
            )
    if PYTHON_VERSION < 3:
        data = json.load(resp)
    else:
        respDecoded = resp.read().decode('utf-8')
        try:
            data = json.loads(respDecoded)
        except Exception as e:
            print(data)
            raise e

    if data['success'] is False:
        raise tamber.error.TamberError(data['error'])
    return data['result']



from distutils.core import setup
setup(
  name = 'tamber',
  packages = ['tamber'],
  version = '0.2.3',
  description = 'Tamber is the easiest way to put head-scratchingly accurate, real time recommendations in your app.  See https://tamber.com for details.',
  author = 'Tamber',
  author_email = 'team@tamber.com',
  url = 'https://github.com/tamber/tamber-python',
  download_url = 'https://pypi.python.org/pypi/tamber', 
  keywords = ['tamber', 'recommendations', 'pytamber', 'backend', 'hosted', 'cloud',
              'collaborative filtering', 'engine', 'suggestion engine', 
              'recommender system', 'suggestions'],
  classifiers = [
        "Development Status :: 5 - Production/Stable",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Programming Language :: Python :: 2",
        "Programming Language :: Python :: 2.6",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.2",
        "Programming Language :: Python :: 3.3",
        "Programming Language :: Python :: 3.4",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: Implementation :: PyPy",
        "Topic :: Software Development :: Libraries :: Python Modules",
    ]
)